# -*- coding: iso-8859-1 -*-

#Copyright (C) Fiz Vazquez vud1@sindominio.net
# vud1@grupoikusnet.com
# Jakinbidea & Grupo Ikusnet Developer
# Modified by dgranda

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import logging
import os
import dateutil
import warnings
from pytrainer.util.color import color_from_hex_string
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.types import TypeDecorator
from sqlalchemy import Integer, Table, Column, ForeignKey

DeclarativeBase = declarative_base()

record_to_equipment = Table('record_equipment', DeclarativeBase.metadata,
                            Column('id', Integer, primary_key=True),
                            Column('equipment_id', Integer,
                                   ForeignKey('equipment.id'),
                                   index=True, nullable=False),
                            Column('record_id', Integer,
                                   ForeignKey('records.id_record'),
                                   index=True, nullable=False))

#Define the tables and their columns that should be in the database
#Obviously, this is not a list but a dict -> TODO: ammend name to avoid confusion!!!
tablesList = {  "records":{     "id_record":"integer primary key autoincrement",
                                        "date":"date",
                                        "sport":"integer",
                                        "distance":"float",
                                        "time":"varchar(200)",
                                        "duration": "integer",
                                        "beats":"float",
                                        "average":"float",
                                        "calories":"int",
                                        "comments":"text",
                                        "gpslog":"varchar(200)",
                                        "title":"varchar(200)",
                                        "upositive":"float",
                                        "unegative":"float",
                                        "maxspeed":"float",
                                        "maxpace":"float",
                                        "pace":"float",
                                        "maxbeats":"float",
                                        "date_time_local":"varchar(40)",
                                        "date_time_utc":"varchar(40)",
                                        },
                        "sports":{      "id_sports":"integer primary key autoincrement",
                                        "name":"varchar(100)",
                                        "weight":"float",
                                        "met":"float",
                                        "max_pace":"integer",
                                        "color":"char(6)",
                                        },
                        "waypoints":{   "id_waypoint":"integer primary key autoincrement",
                                        "lat":"float",
                                        "lon":"float",
                                        "ele":"float",
                                        "comment":"varchar(240)",
                                        "time":"date",
                                        "name":"varchar(200)",
                                        "sym":"varchar(200)",
                                        },
                        "laps":{        "id_lap": "integer primary key autoincrement",
                                        "record": "integer",
                                        "lap_number": "integer",
                                        "elapsed_time": "varchar(20)",
                                        "distance": "float",
                                        "start_lat": "float",
                                        "start_lon": "float",
                                        "end_lat": "float",
                                        "end_lon": "float",
                                        "calories": "int",
                                        "intensity": "varchar(7)",
                                        "laptrigger": "varchar(9)",
                                        "max_speed": "float",
                                        "avg_hr": "int",
                                        "max_hr": "int",
                                        "comments":"text",
                                        },
                        "athletestats": {
                                        "id_athletestat": "integer primary key autoincrement",
                                        "date": "date",
                                        "weight": "float",
                                        "bodyfat": "float",
                                        "restinghr": "integer",
                                        "maxhr": "integer",
                                        },
                        "equipment": {
                                      "id": "integer primary key autoincrement" ,
                                      "description": "varchar (200)",
                                      "active": "boolean",
                                      "life_expectancy": "int",
                                      "prior_usage": "int",
                                      "notes": "text",
                                      },
                        "record_equipment": {
                                     "id": "integer primary key autoincrement",
                                     "record_id": "int",
                                     "equipment_id": "int",
                                     }
                        }

class DDBB:
    def __init__(self, url=None):
        """Initialize database connection, defaulting to SQLite in-memory
if no url is provided"""
        if url:
            self.url = url
        else:
            if 'PYTRAINER_ALCHEMYURL' in os.environ:
                self.url = os.environ['PYTRAINER_ALCHEMYURL']
            else:
                self.url = "sqlite://"
        if self.url.startswith('mysql'):
            self.url = '%s%s' % (self.url, '?charset=utf8')
        self.engine = create_engine(self.url, logging_name='db')
        logging.info("DDBB created with url %s", self.url)

    def get_connection_url(self):
        return self.url

    def connect(self):
        Session = sessionmaker(bind=self.engine)
        self.session = Session()

    def disconnect(self):
        self.session.close()
        self.engine.dispose()

    def select(self,table,cells,condition=None, mod=None):
        warnings.warn("Deprecated call to ddbb.select", DeprecationWarning, stacklevel=2)
        sql = "select %s from %s" %(cells,table)
        if condition is not None:
            sql = "%s where %s" % (sql, condition)
        if mod is not None:
            sql = "%s %s" % (sql, mod)
        return list(self.session.execute(sql))

    def create_tables(self, add_default=True):
        """Initialise the database schema from an empty database."""
        logging.info("Creating database tables")
        from pytrainer.core.sport import Sport
        from pytrainer.core.equipment import Equipment
        from pytrainer.waypoint import Waypoint
        from pytrainer.core.activity import Lap
        from pytrainer.athlete import Athletestat
        DeclarativeBase.metadata.create_all(self.engine)
        if add_default:
            for item in [Sport(name=u"Mountain Bike", weight=0.0, color=color_from_hex_string("0000ff")),
                         Sport(name=u"Bike", weight=0.0, color=color_from_hex_string("00ff00")),
                         Sport(name=u"Run", weight=0.0, color=color_from_hex_string("ffff00"))]:
                self.session.add(item)
            self.session.commit()

    def drop_tables(self):
        """Drop the database schema"""
        DeclarativeBase.metadata.drop_all(self.engine)
                
    def create_backup(self):
        """Create a backup of the current database."""
        try:
            import urlparse
            from urllib import url2pathname
        except ImportError:
            import urllib.parse as urlparse
            from urllib.request import url2pathname
        scheme, netloc, path, params, query, fragment = urlparse.urlparse(self.url)
        if scheme == 'sqlite':
            import datetime
            import gzip
            logging.info("Creating compressed copy of current DB")
            logging.debug('Database path: %s', self.url)
            path = url2pathname(path)
            backup_path = '%s_%s.gz' % (path, datetime.datetime.now().strftime('%Y%m%d_%H%M'))
            with open(path, 'rb') as orig_file:
                with gzip.open(backup_path, 'wb') as backup_file:
                    backup_file.write(orig_file.read())
                    logging.info('Database backup successfully created')

class ForcedInteger(TypeDecorator):
    """Type to force values to int since sqlite doesn't do this"""
    impl = Integer

    def process_bind_param(self, value, dialect):
        if value is None:
            return None
        else:
            return int(value)



# -*- coding: utf-8 -*-
from unittest import TestCase
try:
    from unittest.mock import Mock
except ImportError:
    from mock import Mock
from pytrainer.core.equipment import Equipment, EquipmentService
from pytrainer.gui.equipment import EquipmentStore, EquipmentUi
from pytrainer.lib.ddbb import DDBB
from pytrainer.lib.localization import initialize_gettext

#Copyright (C) Nathan Jones ncjones@users.sourceforge.net
#Copyright (C) Arto Jantunen <viiru@iki.fi>

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

class EquipmentStoreTest(TestCase):
    
    def setUp(self):
        self.mock_equipment_service = Mock(spec=EquipmentService)
        self.mock_equipment_service.get_equipment_usage.return_value = 0
    
    def tearDown(self):
        pass
        
    def test_get_item_id(self):
        equipment = Equipment()
        equipment.id = 1
        self.mock_equipment_service.get_all_equipment.return_value = [equipment]
        equipment_store = EquipmentStore(self.mock_equipment_service)
        iter = equipment_store.get_iter_first()
        self.assertEqual(1, equipment_store.get_value(iter, 0))
        
    def test_get_item_description(self):
        equipment = Equipment()
        equipment.id = 1
        equipment.description = u"item description"
        self.mock_equipment_service.get_all_equipment.return_value = [equipment]
        equipment_store = EquipmentStore(self.mock_equipment_service)
        iter = equipment_store.get_iter_first()
        self.assertEqual("item description", equipment_store.get_value(iter, 1))
        
    def test_get_item_usage_percent(self):
        equipment = Equipment()
        equipment.id = 1
        equipment.life_expectancy = 200
        self.mock_equipment_service.get_all_equipment.return_value = [equipment]
        self.mock_equipment_service.get_equipment_usage.return_value = 100
        equipment_store = EquipmentStore(self.mock_equipment_service)
        iter = equipment_store.get_iter_first()
        self.assertEqual(50, equipment_store.get_value(iter, 2))
        
    def test_get_item_usage_percent_prior_usage(self):
        equipment = Equipment()
        equipment.id = 1
        equipment.life_expectancy = 200
        equipment.prior_usage = 50
        self.mock_equipment_service.get_all_equipment.return_value = [equipment]
        self.mock_equipment_service.get_equipment_usage.return_value = 100
        equipment_store = EquipmentStore(self.mock_equipment_service)
        iter = equipment_store.get_iter_first()
        self.assertEqual(75, equipment_store.get_value(iter, 2))
        
    def test_get_item_usage_percent_zero_usage(self):
        equipment = Equipment()
        equipment.id = 1
        equipment.life_expectancy = 200
        self.mock_equipment_service.get_all_equipment.return_value = [equipment]
        self.mock_equipment_service.get_equipment_usage.return_value = 0
        equipment_store = EquipmentStore(self.mock_equipment_service)
        iter = equipment_store.get_iter_first()
        self.assertEqual(0, equipment_store.get_value(iter, 2))
        
    def test_get_item_usage_percent_usage_exceeds_life_expectancy(self):
        equipment = Equipment()
        equipment.id = 1
        equipment.life_expectancy = 200
        self.mock_equipment_service.get_all_equipment.return_value = [equipment]
        self.mock_equipment_service.get_equipment_usage.return_value = 300
        equipment_store = EquipmentStore(self.mock_equipment_service)
        iter = equipment_store.get_iter_first()
        self.assertEqual(100, equipment_store.get_value(iter, 2), "Progress bar cannot exceed 100%.")
        
        
    def test_get_item_usage_text(self):
        equipment = Equipment()
        equipment.id = 1
        equipment.life_expectancy = 200
        self.mock_equipment_service.get_all_equipment.return_value = [equipment]
        self.mock_equipment_service.get_equipment_usage.return_value = 100
        equipment_store = EquipmentStore(self.mock_equipment_service)
        iter = equipment_store.get_iter_first()
        self.assertEqual("100 / 200", equipment_store.get_value(iter, 3))
        
    def test_get_item_usage_text_rounded(self):
        equipment = Equipment()
        equipment.id = 1
        equipment.life_expectancy = 200
        self.mock_equipment_service.get_all_equipment.return_value = [equipment]
        self.mock_equipment_service.get_equipment_usage.return_value = 100.6
        equipment_store = EquipmentStore(self.mock_equipment_service)
        iter = equipment_store.get_iter_first()
        self.assertEqual("101 / 200", equipment_store.get_value(iter, 3))
        
    def test_get_item_usage_text_prior_usage(self):
        equipment = Equipment()
        equipment.id = 1
        equipment.life_expectancy = 200
        equipment.prior_usage = 50
        self.mock_equipment_service.get_all_equipment.return_value = [equipment]
        self.mock_equipment_service.get_equipment_usage.return_value = 100
        equipment_store = EquipmentStore(self.mock_equipment_service)
        iter = equipment_store.get_iter_first()
        self.assertEqual("150 / 200", equipment_store.get_value(iter, 3))
        
    def test_get_item_usage_text_zero_usage(self):
        equipment = Equipment()
        equipment.id = 1
        equipment.life_expectancy = 200
        self.mock_equipment_service.get_all_equipment.return_value = [equipment]
        self.mock_equipment_service.get_equipment_usage.return_value = 0
        equipment_store = EquipmentStore(self.mock_equipment_service)
        iter = equipment_store.get_iter_first()
        self.assertEqual("0 / 200", equipment_store.get_value(iter, 3))
        
    def test_get_item_usage_text_usage_exceeds_life_expectancy(self):
        equipment = Equipment()
        equipment.id = 1
        equipment.life_expectancy = 200
        self.mock_equipment_service.get_all_equipment.return_value = [equipment]
        self.mock_equipment_service.get_equipment_usage.return_value = 300
        equipment_store = EquipmentStore(self.mock_equipment_service)
        iter = equipment_store.get_iter_first()
        self.assertEqual("300 / 200", equipment_store.get_value(iter, 3))
        
    def test_get_item_active(self):
        equipment = Equipment()
        equipment.id = 1
        equipment.active = False
        self.mock_equipment_service.get_all_equipment.return_value = [equipment]
        equipment_store = EquipmentStore(self.mock_equipment_service)
        iter = equipment_store.get_iter_first()
        self.assertFalse(equipment_store.get_value(iter, 4))
    
    def test_multiple_equipment_items(self):
        equipment1 = Equipment()
        equipment1.id = 1
        equipment2 = Equipment()
        equipment2.id = 2
        self.mock_equipment_service.get_all_equipment.return_value = [equipment1, equipment2]
        equipment_store = EquipmentStore(self.mock_equipment_service)
        iter = equipment_store.get_iter_first()
        iter = equipment_store.iter_next(iter)
        self.assertEqual(2, equipment_store.get_value(iter, 0))
        
class EquipmentUiTest(TestCase):

    def setUp(self):
        initialize_gettext('locale/')
        self.ddbb = DDBB()
        self.ddbb.connect()
        self.ddbb.create_tables()
        self.equipment_service = EquipmentService(self.ddbb)
        self.equipment_ui = EquipmentUi('glade/', self.equipment_service)

    def tearDown(self):
        self.ddbb.disconnect()
        self.ddbb.drop_tables()

    def test_equipment_add(self):
        self.equipment_ui._builder.get_object("entryEquipmentAddDescription").set_text('Test')
        self.equipment_ui._builder.get_object("entryEquipmentAddLifeExpectancy").set_text('500')
        self.equipment_ui._builder.get_object("entryEquipmentAddPriorUsage").set_text('100')
        self.equipment_ui._builder.get_object("checkbuttonEquipmentAddActive").set_active(is_active=True)
        self.equipment_ui._confirm_add_equipment_clicked(None)
        equipment = self.equipment_service.get_equipment_item(1)
        self.assertEqual(equipment.description, 'Test')
        self.assertEqual(equipment.life_expectancy, 500)
        self.assertEqual(equipment.prior_usage, 100)
        self.assertTrue(equipment.active)

    def test_equipment_add_unicode(self):
        self.equipment_ui._builder.get_object("entryEquipmentAddDescription").set_text(u'Test äö')
        self.equipment_ui._builder.get_object("entryEquipmentAddLifeExpectancy").set_text('500')
        self.equipment_ui._builder.get_object("entryEquipmentAddPriorUsage").set_text('100')
        self.equipment_ui._builder.get_object("checkbuttonEquipmentAddActive").set_active(is_active=True)
        self.equipment_ui._confirm_add_equipment_clicked(None)
        equipment = self.equipment_service.get_equipment_item(1)
        self.assertEqual(equipment.description, u'Test äö')



import os
import datetime
import mock

from sqlalchemy.orm.session import make_transient

from gi.repository import Gtk
from unittest import TestCase
from pytrainer.lib.ddbb import DDBB
from pytrainer.lib.localization import initialize_gettext, gtk_str
from pytrainer.core.sport import SportService
from pytrainer.environment import Environment
from pytrainer.profile import Profile
from pytrainer.record import Record
from pytrainer.core.activity import Activity
initialize_gettext('locale/')

from pytrainer.gui.windowmain import Main
from pytrainer.lib.listview import ListSearch
from pytrainer.lib.uc import UC

#Copyright (C) Arto Jantunen <viiru@iki.fi>

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

class ListviewTest(TestCase):

    def setUp(self):
        env = Environment()
        env.data_path = os.curdir
        env.create_directories()
        self.main = mock.Mock()
        self.main.ddbb = DDBB()
        self.main.ddbb.connect()
        self.main.ddbb.create_tables()
        self.main.profile = Profile()
        self.main.uc = UC()
        self.sport_service = SportService(self.main.ddbb)
        non_linear_sport = self.sport_service.get_sport(1)
        self.main.ddbb.session.delete(non_linear_sport)
        self.main.ddbb.session.commit()
        make_transient(non_linear_sport)
        non_linear_sport.id = None
        self.main.ddbb.session.add(non_linear_sport)
        self.main.ddbb.session.commit()
        self.main.record = Record(self.sport_service, parent=self.main)
        self.parent = Main(self.sport_service, parent=self.main)
        self.main.ddbb.session.add(Activity(sport=non_linear_sport,
                                            date=datetime.datetime(2018, 5, 6, 10, 0, 0),
                                            duration=1000, distance=25))
        self.main.ddbb.session.add(Activity(sport=non_linear_sport,
                                            date=datetime.datetime(2018, 1, 6, 10, 0, 0),
                                            duration=10000, distance=150))
        self.main.ddbb.session.add(Activity(sport=self.sport_service.get_sport(2),
                                            date=datetime.datetime(2018, 1, 6, 10, 0, 0),
                                            duration=10000, distance=100))
        self.main.ddbb.session.commit()

    def tearDown(self):
        self.main.ddbb.disconnect()
        self.main.ddbb.drop_tables()

    def test_listsearch_all(self):
        self.assertEqual(len(list(self.main.record.getRecordListByCondition(None))), 3)

    def test_listsearch_defaults(self):
        self.assertEqual(len(list(self.main.record.getRecordListByCondition(self.parent.listsearch.condition))), 3)

    def test_listsearch_sport(self):
        self.parent.lsa_sport.set_active(3)
        active = gtk_str(self.parent.lsa_sport.get_active_text())
        by_sport = list(self.main.record.getRecordListByCondition(self.parent.listsearch.condition))
        self.assertEqual(len(by_sport), 2)
        self.assertEqual(by_sport[0].sport.name, active)

    def test_listsearch_distance(self):
        self.parent.lsa_distance.set_active(4)
        self.assertEqual(len(list(self.main.record.getRecordListByCondition(self.parent.listsearch.condition))), 1)

    def test_listsearch_duration(self):
        self.parent.lsa_duration.set_active(1)
        self.assertEqual(len(list(self.main.record.getRecordListByCondition(self.parent.listsearch.condition))), 1)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#Copyright (C) Fiz Vazquez vud1@sindominio.net

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

# Added to support python installations older than 2.6
from __future__ import with_statement

import os
import sys
import logging
import fnmatch
import subprocess
from io import BytesIO
import traceback
import dateutil.parser
import time

from lxml import etree
from pytrainer.lib.xmlUtils import XMLParser
from datetime import date, timedelta, datetime
from dateutil.tz import * # for tzutc()
from pytrainer.core.activity import Activity

class garmintools_full():
    """ Plugin to import from a Garmin device using garmintools
            Checks each activity to see if any entries are in the database with the same start time
            Creates GPX files for each activity not in the database

            Note: using lxml see http://codespeak.net/lxml
    """
    def __init__(self, parent = None, validate=False):
        self.parent = parent
        self.pytrainer_main = parent.pytrainer_main
        self.confdir = self.pytrainer_main.profile.confdir
        self.tmpdir = self.pytrainer_main.profile.tmpdir
        # Tell garmintools where to save retrieved data from GPS device
        os.environ['GARMIN_SAVE_RUNS']=self.tmpdir
        self.data_path = os.path.dirname(__file__)
        self.validate = validate
        self.sport = self.getConfValue("Force_sport_to")
        self.deltaDays = self.getConfValue("Not_older_days")
        if self.deltaDays is None:
            logging.info("Delta days not set, retrieving complete history, defaulting to 0")
            self.deltaDays = 0
        #so far hardcoded to False - dg 20100104
        #self.legacyComp = self.getConfValue("Legacy_comparison")
        self.maxGap = self.getConfValue("Max_gap_seconds")
        if self.maxGap is None:
            logging.info("No gap defined, strict comparison")
            self.maxGap = 0

    def getConfValue(self, confVar):
        info = XMLParser(self.data_path+"/conf.xml")
        code = info.getValue("pytrainer-plugin","plugincode")
        plugindir = self.pytrainer_main.profile.plugindir
        if not os.path.isfile(plugindir+"/"+code+"/conf.xml"):
            value = None
        else:
            info = XMLParser(plugindir+"/"+code+"/conf.xml")
            value = info.getValue("pytrainer-plugin",confVar)
        return value

    def error_dialog(self, msg):
        #TODO Remove Zenity below
        subprocess.call(["zenity", "--error", "--text='%s'" % msg])

    def run(self):
        logging.debug(">>")
        importFiles = []
        if self.checkLoadedModule():
            numError = self.getDeviceInfo()
            if numError >= 0:
                #TODO Remove Zenity below
                outgps = subprocess.call(
                        "garmin_save_runs -v "
                        "| zenity --progress --pulsate "
                        "         --text='Loading Data' --auto-close",
                        shell=True)
                if outgps == 0:
                    # now we should have a lot of gmn (binary) files under $GARMIN_SAVE_RUNS
                    foundFiles = self.searchFiles(self.tmpdir, "gmn")
                    logging.info("Retrieved "+str(len(foundFiles))+" entries from GPS device")
                    # Trying to minimize number of files to dump
                    if int(self.deltaDays) > 0:
                        selectedFiles = self.discardOld(foundFiles)
                    else:
                        logging.info("Retrieving complete history from GPS device")
                        selectedFiles = foundFiles
                    if len(selectedFiles) > 0:
                        logging.info("Dumping "+str(len(selectedFiles))+" binary files found")
                        dumpFiles = self.dumpBinaries(selectedFiles)
                        self.listStringDBUTC = self.pytrainer_main.ddbb.session.query(Activity)
                        if self.maxGap > 0:
                            logging.info("Starting import. Comparison will be made with "+str(self.maxGap)+" seconds interval")
                        else:
                            logging.info("Starting import. Comparison will be strict")
                        importFiles = self.importEntries(dumpFiles)
                    else:
                        logging.info("No new entries to add")
                else:
                    logging.error("Error when retrieving data from GPS device")
            else:
                if numError == -1:
                    self.error_dialog("No Garmin device found\n"
                                      "Check your configuration")
                elif numError == -2:
                    self.error_dialog("Can not find garmintools binaries\n"
                                      "Check your configuration")
        else: #No garmin device found
            self.error_dialog("Can not handle Garmin device (wrong "
                              "module loaded)\n"
                              "Check your configuration")
        logging.info("Entries to import: "+str(len(importFiles)))
        logging.debug("<<")
        return importFiles

    def discardOld(self, listEntries):
        logging.debug(">>")
        tempList = []
        logging.info("Discarding entries older than "+str(self.deltaDays)+" days")
        limit = datetime.now() - timedelta(days = int(self.deltaDays))
        for entry in listEntries:
            filename = os.path.split(entry)[1].rstrip(".gmn")
            filenameDateTime = datetime.strptime(filename,"%Y%m%dT%H%M%S")
            logging.debug("Entry time: "+str(filenameDateTime)+" | limit: "+str(limit))
            if filenameDateTime < limit:
                logging.debug("Discarding old entry: "+str(filenameDateTime))
            else:
                tempList.append(entry)
        logging.debug("<<")
        return tempList

    def importEntries(self, entries):
        # modified from garmintools plugin written by jb
        logging.debug(">>")
        logging.debug("Selected files: "+str(entries))
        importfiles = []
        for filename in entries:
            if self.valid_input_file(filename):
                #Garmin dump files are not valid xml - need to load into a xmltree
                #read file into string
                with open(filename, 'r') as f:
                    xmlString = f.read()
                # Double check encoding from dump files. ASCII?
                fileString = BytesIO(b"<root>" + xmlString + b"</root>")
                #parse string as xml
                try:
                    tree = etree.parse(fileString)
                    #if not self.inDatabase(tree, filename):
                    if not self.entryExists(tree, filename):
                        sport = self.getSport(tree)
                        gpxfile = "%s/garmintools-%d.gpx" % (self.tmpdir, len(importfiles))
                        self.createGPXfile(gpxfile, tree)
                        importfiles.append((gpxfile, sport))
                    else:
                        logging.debug("%s already present. Skipping import." % (filename,) )
                except:
                    logging.error('Error parsing entry '+ str(filename))
                    traceback.print_exc()
            else:
                logging.error("File %s failed validation" % (filename))
        logging.debug("<<")
        return importfiles

    def valid_input_file(self, filename):
        """ Function to validate input file if requested"""
        if not self.validate:  #not asked to validate
            logging.debug("Not validating %s" % (filename) )
            return True
        else:
            logging.debug("Cannot validate garmintools dump files yet")
            return True
            '''xslfile = os.path.realpath(self.parent.parent.data_path)+ "/schemas/GarminTrainingCenterDatabase_v2.xsd"
            from lib.xmlValidation import xmlValidator
            validator = xmlValidator()
            return validator.validateXSL(filename, xslfile)'''

    def entryExists(self, tree, filename):
        logging.debug(">>")
        stringStartDatetime = self.detailsFromFile(tree) # this time is localtime! (with timezone offset)
        exists = False
        if stringStartDatetime is not None:
            startDatetime = dateutil.parser.parse(stringStartDatetime)
            # converting to utc for proper comparison with date_time_utc
            stringStartUTC = startDatetime.astimezone(tzutc()).strftime("%Y-%m-%dT%H:%M:%SZ")
            if self.checkDupe(stringStartUTC, self.listStringDBUTC, int(self.maxGap)):
                exists = True
            else:
                logging.info("Marking "+str(filename)+" | "+str(stringStartUTC)+" to import")
                exists = False
        else:
            logging.debug("Not able to find start time, please check "+str(filename))
            exists = True # workaround for old/not correct entries (will crash at some point during import process otherwise)
        logging.debug("<<")
        return exists

    def checkDupe(self, stringStartUTC, listStringStartUTC, gap):
        """ Checks if there is any startUTC in DB between provided startUTC plus a defined gap:
                Check for same day (as baselined to UTC)
                startDatetime + delta (~ 3 mins) >= listDatetime[x]
                args:
                        stringStartUTC
                        listStringStartUTC
                        gap
                returns: True if any coincidence is found. False otherwise"""
        logging.debug(">>")
        found = False
        if gap > 0:
            # Retrieve date from 2010-01-14T11:34:49Z
            stringStartDate = stringStartUTC[0:10]
            for entry in listStringStartUTC:
                #logging.debug("start: "+str(startDatetime)+" | entry: "+str(entry)+" | gap: "+str(datetimePlusDelta))
                if entry.date_time_utc.startswith(stringStartDate):
                    deltaGap = timedelta(seconds=gap)
                    datetimeStartUTC = datetime.strptime(stringStartUTC,"%Y-%m-%dT%H:%M:%SZ")
                    datetimeStartUTCDB = datetime.strptime(entry.date_time_utc, "%Y-%m-%dT%H:%M:%SZ")
                    datetimePlusDelta = datetimeStartUTC + deltaGap
                    if datetimeStartUTC <= datetimeStartUTCDB and datetimeStartUTCDB <= datetimePlusDelta:
                        found = True
                        logging.debug("Found: "+str(stringStartUTC)+" <= "+entry.date_time_utc+" <= "+str(datetimePlusDelta))
                        break
        else:
            if (stringStartUTC,) in listStringStartUTC: # strange way to store results from DB
                found = True
        logging.debug("<<")
        return found

    def getSport(self, tree):
        #return sport from file or overide if present
        if self.sport:
            return self.sport
        root = tree.getroot()
        sportElement = root.find(".//run")
        try:
            sport = sportElement.get("sport")
            sport = sport.capitalize()
        except:
            sport = "import"
        return sport

    def detailsFromFile(self, tree):
        root = tree.getroot()
        #Find first point
        pointElement = root.find(".//point")
        if pointElement is not None:
            stringStartDatetime = pointElement.get("time")
            return stringStartDatetime
        return None

    def createGPXfile(self, gpxfile, tree):
        """ Function to transform a Garmintools dump file to a valid GPX+ file
        """
        xslt_doc = etree.parse(self.data_path+"/translate.xsl")
        transform = etree.XSLT(xslt_doc)
        result_tree = transform(tree)
        result_tree.write(gpxfile, xml_declaration=True, encoding='UTF-8')

    def dumpBinaries(self, listFiles):
        logging.debug(">>")
        dumpFiles=[]
        for filename in listFiles:
            outdump = filename.replace('.gmn', '.dump')
            logging.debug("outdump: "+str(outdump))
            result = subprocess.call(
                    "garmin_dump %s > %s" % (filename, outdump))
            if result == 0:
                dumpFiles.append(outdump)
            else:
                logging.error("Error when creating dump of %s: %d" %
                              (str(filename), result))
        logging.debug("<<")
        return dumpFiles

    def searchFiles(self, rootPath, extension):
        logging.debug(">>")
        foundFiles=[]
        logging.debug("rootPath: "+str(rootPath))
        process = subprocess.Popen(
                ["find", rootPath, "-name", "*.%s" % extension],
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE)
        stdout, stderr = process.communicate()
        if process.returncode == 0:
            foundFiles = stdout.splitlines()
            #logging.debug("Found files: "+str(foundFiles))
            logging.info ("Found files: "+str(len(foundFiles)))
        else:
            logging.error("Not able to locate files from GPS: %s" % 
                          str(process.returncode))
        logging.debug("<<")
        return foundFiles

    def getDeviceInfo(self):
        logging.debug(">>")
        process = subprocess.Popen('garmin_get_info',
                                   stdout=subprocess.PIPE,
                                   stderr=subprocess.PIPE)
        stdout, stderr = process.communicate()
        logging.debug("Returns "+str(stdout))
        numError = 0
        if process.returncode == 0:
            if stdout != "garmin unit could not be opened!":
                try:
                    xmlString = result[1].rstrip()
                    xmlString_2 = ' '.join(xmlString.split())
                    tree = etree.fromstring(xmlString_2)
                    description = self.getProductDesc(tree)
                    if description is not None:
                        logging.info("Found "+str(description))
                    else:
                        raise Exception
                except:
                    logging.error("Not able to identify GPS device. Continuing anyway...")
                    logging.debug("Traceback: %s" % traceback.format_exc())
                    pass
            else:
                logging.error(result[1])
                numError = -1
        else:
            logging.error("Can not find garmintools binaries, please check your installation")
            numError = -2
        logging.debug("<<")
        return numError

    def getProductDesc(self, tree):
        desc = tree.findtext(".//product_description")
        return desc

    def checkLoadedModule(self):
        try:
            outmod = subprocess.Popen('lsmod | grep garmin_gps',
                                      stdout=subprocess.PIPE,
                                      stderr=subprocess.PIPE,
                                      shell=True)
            stdout, stderr = outmod.communicate()
            # stdout is empty if no garmin_gps module is loaded
            return stdout != ''
        except:
            return False

    def createUserdirBackup(self):
        logging.debug('>>')
        result = subprocess.call(
                ['tar',
                 '-cvzf',
                 '%s/pytrainer_%s.tar.gz' % (os.environ['HOME'],
                                             time.strftime('%Y%m%d_%H%M')),
                 self.confdir])
        if result != 0:
            raise Exception("Copying current user directory does not work, "
                              "error #"+str(result))
        else:
            logging.info('User directory backup successfully created')
        logging.debug('<<')



# -*- coding: iso-8859-1 -*-

#Copyright (C) Fiz Vazquez vud1@sindominio.net
# Modified by dgranda

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import logging
import os
from lxml import etree

from pytrainer.lib.xmlUtils import XMLParser
from pytrainer.lib.date import getDateTime
from pytrainer.core.activity import Activity
from sqlalchemy.orm import exc

class garmintcxv1():
    def __init__(self, parent = None, data_path = None):
        logging.debug("init")
        self.parent = parent
        self.pytrainer_main = parent.parent
        self.tmpdir = self.pytrainer_main.profile.tmpdir
        self.main_data_path = data_path
        self.data_path = os.path.dirname(__file__)
        self.xmldoc = None
        self.activitiesSummary = []
        self.activities = []
        self.sportsList = ("Running", "Biking", "Other", "MultiSport")

    def getXmldoc(self):
        ''' Function to return parsed xmlfile '''
        return self.xmldoc

    def getFileType(self):
        return _("Garmin training center database file version 1")

    def getActivitiesSummary(self):
        return self.activitiesSummary

    def getDetails(self, activity, startTime):
        points = activity.findall(".//{http://www.garmin.com/xmlschemas/TrainingCenterDatabase/v1}Trackpoint")
        while True:
            lastPoint = points[-1]
            try:
                time = lastPoint.find(".//{http://www.garmin.com/xmlschemas/TrainingCenterDatabase/v1}Time")
                time = time.text
                break
            except:
                #Try again without the last point (i.e work from end until find time)
                points = points[:-1]
                continue
        return getDateTime(time)[0]-startTime[0]

    def testFile(self, filename):
        logging.debug('>>')
        logging.debug("Testing " + filename)
        try:
            #parse filename as xml
            xmldoc = etree.parse(filename)
            #Parse XML schema
            xmlschema_doc = etree.parse(self.main_data_path+"schemas/GarminTrainingCenterDatabase_v1-gpsbabel.xsd")
            xmlschema = etree.XMLSchema(xmlschema_doc)
            if (xmlschema.validate(xmldoc)):
                #Valid file
                self.xmldoc = xmldoc
                #Possibly multiple entries in file
                self.activities = self.getActivities(xmldoc)
                for (sport, activities) in self.activities:
                    logging.debug("Found %d tracks for %s sport in %s" % (len(activities), sport, filename))
                    for activity in activities:
                        startTime = getDateTime(self.getStartTimeFromActivity(activity))
                        inDatabase = self.inDatabase(activity, startTime)
                        duration  = self.getDetails(activity, startTime)
                        distance = ""
                        index = "%d:%d" % (self.activities.index((sport, activities)), activities.index(activity))
                        self.activitiesSummary.append( (index,
                                                                                        inDatabase,
                                                                                        startTime[1].strftime("%Y-%m-%dT%H:%M:%S"),
                                                                                        distance ,
                                                                                        str(duration),
                                                                                        sport,
                                                                                        ) )
                #print self.activitiesSummary
                return True
        except:
            #Not valid file
            return False
        return False

    def getActivities(self, tree):
        """ Function to return all the tracks in a Garmin Training Center v1 file
        """
        result = []
        root = tree.getroot()
        for sport in self.sportsList:
            try:
                sportLevel = root.find(".//{http://www.garmin.com/xmlschemas/TrainingCenterDatabase/v1}%s" % sport)
                tracks = sportLevel.findall(".//{http://www.garmin.com/xmlschemas/TrainingCenterDatabase/v1}Track")
                result.append((sport, tracks))
            except:
                print("No entries for sport %s" % sport)
        return result

    def inDatabase(self, activity, startTime):
        #comparing date and start time (sport may have been changed in DB after import)
        time = startTime
        if time is None:
            return False
        time = time[0].strftime("%Y-%m-%dT%H:%M:%SZ")
        try:
            self.parent.parent.ddbb.session.query(Activity).filter(Activity.date_time_utc == time).one()
            return True
        except exc.NoResultFound:
            return False

    def getStartTimeFromActivity(self, activity):
        timeElement = activity.find(".//{http://www.garmin.com/xmlschemas/TrainingCenterDatabase/v1}Time")
        if timeElement is None:
            return None
        else:
            return timeElement.text

    def getGPXFile(self, ID, file_id):
        """
                Generate GPX file based on activity ID

                Returns (sport, GPX filename)
        """
        sport = None
        gpxFile = None
        #index = "%d:%d" % (self.activities.index((sport, activities)), activities.index(activity))
        sportID, activityID = ID.split(':')
        sportID = int(sportID)
        activityID = int(activityID)
        sport, activities = self.activities[sportID]
        activitiesCount = len(self.activities)
        if activitiesCount > 0 and activityID < activitiesCount:
            gpxFile = "%s/garmin-tcxv1-%s-%d.gpx" % (self.tmpdir, file_id, activityID)
            activity = activities[activityID]
            self.createGPXfile(gpxFile, activity)
        return sport, gpxFile

    def createGPXfile(self, gpxfile, activity):
        """ Function to transform a Garmin Training Center v2 Track to a valid GPX+ file
        """
        xslt_doc = etree.parse(self.data_path+"/translate_garmintcxv1.xsl")
        transform = etree.XSLT(xslt_doc)
        #xml_doc = etree.parse(filename)
        xml_doc = activity
        result_tree = transform(xml_doc)
        result_tree.write(gpxfile, xml_declaration=True, encoding='UTF-8')



# -*- coding: iso-8859-1 -*-

#Copyright (C) Fiz Vazquez vud1@sindominio.net

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from pytrainer.extension import Extension
from pytrainer.lib.fileUtils import fileUtils
from gi.repository import Gtk
import logging
import os
import re
from gi.repository import WebKit2

class WaypointEditor:
    def __init__(self, data_path = None, vbox = None, waypoint=None, parent=None):
        logging.debug(">>")
        self.data_path = data_path
        self.extension = Extension()
        self.wkview = WebKit2.WebView()
        self.wkview.connect('notify::title', self.handle_title_changed)
        scrolled_window = Gtk.ScrolledWindow()
        scrolled_window.add(self.wkview)
        vbox.pack_start(scrolled_window, True, True, 0)
        vbox.show_all()
        self.htmlfile = ""
        self.waypoint=waypoint
        self.pytrainer_main=parent
        logging.debug("<<")

    def handle_title_changed(self, *args):
        title = self.wkview.get_title()
        if title == None:
            return
        logging.debug("Received title: "+ title)
        m = re.match("call:([a-zA-Z]*)[(](.*)[)]", title)
        if m:
            fname = m.group(1)
            args = m.group(2)
            if fname == "addWaypoint":
                am = re.match("([+-]?[0-9]+[.][0-9]+),([+-]?[0-9]+[.][0-9]+)", args)
                if am:
                    lon, lat = am.group(1), am.group(2)
                    lon, lat = float(lon), float(lat)
                    id_waypoint = self.waypoint.addWaypoint(lon, lat, "NEW WAYPOINT")
                    self.pytrainer_main.refreshWaypointView(default_waypoint=id_waypoint)
                else:
                    raise ValueError("Error parsing addWaypoint parameters: %s" % args)
            elif fname == "updateWaypoint":
                am = re.match("([+-]?[0-9]+[.][0-9]+),([+-]?[0-9]+[.][0-9]+),([0-9]*)", args)
                if am:
                    lon, lat, id_waypoint = am.group(1), am.group(2), am.group(3)
                    try:
                        lon, lat, id_waypoint = float(lon), float(lat), int(id_waypoint)
                    except ValueError as e:
                        logging.error("Error parsing addWaypoint parameters: %s %s", args, e)
                    retorno = self.waypoint.getwaypointInfo(id_waypoint)
                    if retorno:
                        name, comment, sym = retorno[0][5], retorno[0][3], retorno[0][6]
                        self.waypoint.updateWaypoint(id_waypoint, lat, lon, name, comment, sym)
                        self.pytrainer_main.refreshWaypointView(default_waypoint=id_waypoint)
                    else:
                        raise KeyError("Unknown waypoint id %d", id_waypoint)
                else:
                    raise ValueError("Error parsing addWaypoint parameters: %s" % args)
            else:
                raise ValueError("Unexpected function name %s" % fname)
        return False

    def drawMap(self):
        logging.debug(">>")
        #self.createHtml()
        tmpdir = self.pytrainer_main.profile.tmpdir
        htmlfile = tmpdir+"/waypointeditor.html"
        logging.debug("HTML file: "+str(htmlfile))
        self.wkview.load_uri("file://"+htmlfile)
        logging.debug("<<")

    def createHtml(self,default_waypoint=None):
        logging.debug(">>")
        tmpdir = self.pytrainer_main.profile.tmpdir
        filename = tmpdir+"/waypointeditor.html"

        points = self.waypoint.getAllWaypoints()
        londef = 0
        latdef = 0
        content = """

<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml"  xmlns:v="urn:schemas-microsoft-com:vml">
<head>
<meta http-equiv="content-type" content="text/html; charset=utf-8"/>
<title>edit waypoints</title>

<script id="googleapiimport" src="https://maps.google.com/maps/api/js"
    type="text/javascript"></script>
<script type="text/javascript">
"""
        i = 0
        arrayjs = ""
        if default_waypoint is None and points:
            default_waypoint = points[0][0]
        for point in points:
            if point[0] == default_waypoint:
                londef = point[2]
                latdef = point[1]
            content += "lon = '%f';\n"%point[2]
            content += "lat = '%f';\n"%point[1]
            content += "name = '%s';\n"%point[6]
            content += "description = '%s';\n"%point[4]
            content += "sym = '%s';\n"%point[7]
            content += "id = '%d';\n"%point[0]
            content += """waypoint%d = Array (lon,lat,name,description,sym,id);\n"""%i
            if i>0:
                arrayjs+=","
            arrayjs +="waypoint%d"%i
            i = i+1
        content += """waypointList = Array (%s);\n""" %arrayjs
        content += """
is_addmode = 0;
var map;
//<![CDATA[

function addWaypoint(lon,lat) {
        document.title = "call:addWaypoint(" + lon + "," + lat + ")";
        }

function updateWaypoint(lon,lat,id) {
        document.title = "call:updateWaypoint(" + lon + "," + lat + "," + id + ")";
        }

function createMarker(waypoint) {
        var lon = waypoint[0];
        var lat = waypoint[1];
        var id = waypoint[5];
        var sym = waypoint[4];

        var point = new google.maps.LatLng(lat,lon);
        var text = "<b>"+waypoint[2]+"</b><br/>"+waypoint[3];

        if (sym=="Summit") {
                var icon_image = \""""+os.path.abspath(self.data_path)+"""/glade/summit.png\";
        } else {
                var icon_image = \""""+os.path.abspath(self.data_path)+"""/glade/waypoint.png\";
        }
        var icon = new google.maps.MarkerImage(icon_image,\n
            new google.maps.Size(32, 32),
            new google.maps.Point(0,0),
            new google.maps.Point(16, 16)
        );\n

        var markerD = new google.maps.Marker({
          position: point,
          map: map,
          icon: icon,
          draggable: true,
        });

        markerD.setDraggable(true);

        google.maps.event.addListener(markerD, "mouseup", function(){
                position = markerD.getPosition();
                updateWaypoint(position.lng(),position.lat(),id);
        });
        return markerD;
}

function load() {
        //Dibujamos el mapa
        var myOptions = {
            zoom: 8,
            // center: centerlatlng,
            mapTypeControl: true,
            scaleControl: true,
            largeMapControl: true,
            overviewMapControl: true,
            overviewMapControlOptions: {opened: true},
            mapTypeId: google.maps.MapTypeId.ROADMAP,
        };
        map = new google.maps.Map(document.getElementById("map_canvas"), myOptions);
"""
        if londef != 0:
            content +="""
                    lon = %s;
                    lat = %s;
                    """ %(londef,latdef)
        else:
            content += """
                   lon = 0;
                   lat = 0;
                   """
        content +="""
        map.setCenter(new google.maps.LatLng(lat, lon), 11);

        //Dibujamos los waypoints
        for (i=0; i<waypointList.length; i++){
                createMarker(waypointList[i]);
                map.setOptions({draggable: true});
        }

        //Preparamos los eventos para anadir nuevos waypoints
        google.maps.event.addListener(map, "click", function(event) {
                if (is_addmode==1){
                        map.setOptions({draggable: true});
                        var lon = event.latLng.lng();
                        var lat = event.latLng.lat();

                        var waypoint_id = addWaypoint(lon,lat);
                        var waypoint = Array (lon,lat,"","","",waypoint_id);
                        createMarker(waypoint);
                        is_addmode = 0;
                }
        });
}

function addmode(){
        is_addmode = 1;
        map.setOptions({draggable: false});
}

//]]>
</script>
<style>
.form {
position: absolute;
top: 200px;
left: 300px;
background: #ffffff;
}
</style>

</head>
<body onload="load()" style="cursor:crosshair" border=0>
        <div id="map_canvas" style="width: 100%; height: 460px; top: 0px; left: 0px"></div>
        <div id="addButton" style="position: absolute; top: 32px;left: 86px;">
                <input type="button" value="New Waypoint" onclick="javascript:addmode();">
        </div>


</body>
</html>
"""
        file = fileUtils(filename,content)
        file.run()
        logging.debug("<<")



#!/usr/bin/env python
# -*- coding: iso-8859-1 -*-

#Copyright (C) Fiz Vazquez vud1@sindominio.net

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import os, sys
import logging
from lxml import etree
from pytrainer.lib.xmlUtils import XMLParser
from pytrainer.core.activity import Activity
from sqlalchemy.orm import exc

import subprocess

class garminhr():
	""" Plugin to import from a Garmin device using gpsbabel
		Checks each activity to see if any entries are in the database with the same start time
		Creates GPX files for each activity not in the database

		Note: using lxml see http://codespeak.net/lxml
	"""
	def __init__(self, parent = None, validate=False):
		self.parent = parent
		self.pytrainer_main = parent.pytrainer_main
		self.tmpdir = self.pytrainer_main.profile.tmpdir
		self.data_path = os.path.dirname(__file__)
		self.validate = validate
		self.input_dev = self.getConfValue("device")
		self.sport = self.getConfValue("Force_sport_to")

	def getConfValue(self, confVar):
		info = XMLParser(self.data_path+"/conf.xml")
		code = info.getValue("pytrainer-plugin","plugincode")
		plugindir = self.pytrainer_main.profile.plugindir
		if not os.path.isfile(plugindir+"/"+code+"/conf.xml"):
			value = None
		else:
			info = XMLParser(plugindir+"/"+code+"/conf.xml")
			value = info.getValue("pytrainer-plugin",confVar)
		return value

	def error_dialog(self, msg):
		#TODO Remove Zenity below
		subprocess.call(["zenity", "--error",
		                 "--text='%s'" % msg])

	def run(self):
		logging.debug(">>")
		importfiles = []
		no_device_msg = ("Can not handle Garmin device\n"
		                 "Check your configuration\n"
		                 "Current USB port is set "
		                 "to:\t %s" % self.input_dev)

		if self.garminDeviceExists():
			try:
				gpsbabelOutputFile = "%s/file.gtrnctr" % (self.tmpdir)
				#TODO Remove Zenity below
				outgps = subprocess.Popen(
				        "gpsbabel -t -i garmin -f %s -o gtrnctr -F %s "
				        "| zenity --progress --pulsate --text='Loading Data' "
				        "    auto-close" % (self.input_dev, gpsbabelOutputFile),
				        stdout=subprocess.PIPE,
				        stderr=subprocess.PIPE,
				        shell=True)
				stdout, stderr = outgps.communicate()
				if outgps.returncode == 0:
					if stdout == "Found no Garmin USB devices.": # check localizations
						error_msg = "GPSBabel found no Garmin USB devices"
						logging.error(error_msg)
						self.error_dialog(error_msg)
					else: #gpsbabel worked - now process file...
						if self.valid_input_file(gpsbabelOutputFile):
							for (sport, tracks) in self.getTracks(gpsbabelOutputFile):
								logging.debug("Found %d tracks for %s sport in %s" % (len(tracks), sport, gpsbabelOutputFile))
								count = 0
								for track in tracks: #can be multiple tracks
									if self.shouldImport(track):
										count += 1
										gpxfile = "%s/garminhrfile%d.gpx" % (self.tmpdir, len(importfiles))
										self.createGPXfile(gpxfile, track)
										if self.sport: #Option to overide sport is set
											importfiles.append((gpxfile, self.sport))
										else: #Use sport from file
											importfiles.append((gpxfile, sport))
								logging.debug("Importing %d of %d tracks for sport %s" % (count, len(tracks), sport) )
						else:
							logging.info("File %s failed validation" % (gpsbabelOutputFile))
			except Exception:
				self.dialog_msg(no_device_msg)
				print(sys.exc_info()[0])
		else: #No garmin device found
			self.error_dialog(no_device_msg)
		logging.debug("<<")
		return importfiles

	def garminDeviceExists(self):
		try:
			process = subprocess.Popen('lsmod | grep garmin_gps',
			                           stdout=subprocess.PIPE,
			                           stderr=subprocess.PIPE,
			                           shell=True)
			stdout, stderr = process.communicate()
			# stdout is empty if no garmin_gps module is loaded
			if stdout != '':
				self.input_dev = "usb:"
				return True
			else:
				return False
		except:
			return False

	def valid_input_file(self, filename):
		""" Function to validate input file if requested"""
		if not self.validate: #not asked to validate
			logging.debug("Not validating %s" % (filename) )
			return True
		else: #Validate TCXv1, note are validating against gpsbabels 'broken' result...
			xslfile = os.path.realpath(self.pytrainer_main.data_path)+ "/schemas/GarminTrainingCenterDatabase_v1-gpsbabel.xsd"
			from lib.xmlValidation import xmlValidator
			validator = xmlValidator()
			return validator.validateXSL(filename, xslfile)

	def getTracks(self, filename):
		""" Function to return all the tracks in a Garmin Training Center v1 file
		"""
		sportsList = ("Running", "Biking", "Other", "MultiSport")
		result = []
		tree = etree.ElementTree(file=filename)
		root = tree.getroot()
		for sport in sportsList:
			try:
				sportLevel = root.find(".//{http://www.garmin.com/xmlschemas/TrainingCenterDatabase/v1}%s" % sport)
				tracks = sportLevel.findall(".//{http://www.garmin.com/xmlschemas/TrainingCenterDatabase/v1}Track")
				result.append((sport, tracks))
			except:
				print("No entries for sport %s" % sport)
		return result

	def shouldImport(self, track):
		""" Function determines whether a track should be imported or not
			Currently using time only
		"""
		timeElement = track.find(".//{http://www.garmin.com/xmlschemas/TrainingCenterDatabase/v1}Time")
		if timeElement is None:
			#print (etree.tostring(track, pretty_print=True))
			logging.debug("Error no time found in track")
			return False
		else:
			time = timeElement.text
			try:
				self.pytrainer_main.ddbb.session.query(Activity).filter(Activity.date_time_utc == time).one()
				return False
			except exc.NoResultFound:
				return True

	def createGPXfile(self, gpxfile, track):
		""" Function to transform a Garmin Training Center v1 Track to a valid GPX+ file
		"""
		xslt_doc = etree.parse(self.data_path+"/translate.xsl")
		transform = etree.XSLT(xslt_doc)
		result_tree = transform(track)
		result_tree.write(gpxfile, xml_declaration=True, encoding='UTF-8')






# -*- coding: iso-8859-1 -*-

#Copyright (C) Fiz Vazquez vud1@sindominio.net
# vud1@grupoikusnet.com
# Jakinbidea & Grupo Ikusnet Developer

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import time
import datetime
import calendar
import dateutil.parser
from dateutil.tz import tzutc, tzlocal
import logging

from pytrainer.platform import get_platform
from pytrainer.lib.localization import locale_str

def second2time(seconds):
    if not seconds:
        return 0,0,0
    hours = seconds // (60*60)
    seconds %= (60*60)
    minutes = seconds // 60
    seconds %= 60
    return hours,minutes,seconds

def time2second(time):
    hour,min,sec = time
    return int(sec)+(int(min)*60)+(int(hour)*3600)

def getLocalTZ():
    ''' Returns string representation of local timezone'''
    return datetime.datetime.now(tzlocal()).tzname()

def time2string(date):
    return "%0.4d-%0.2d-%0.2d" %(int(date[0]),int(date[1]),int(date[2]))

def getNameMonth(date):
    day, daysInMonth = calendar.monthrange(date.year, date.month)
    monthName = locale_str(calendar.month_name[date.month])
    return monthName, daysInMonth

def unixtime2date(unixtime):
    tm = time.gmtime(unixtime)
    year = tm[0]
    month = tm[1]
    day = tm[2]
    return "%0.4d-%0.2d-%0.2d" %(year,month,day)

def getDateTime(time_):
    # Time can be in multiple formats
    # - zulu            2009-12-15T09:00Z
    # - local ISO8601   2009-12-15T10:00+01:00
    try:
        dateTime = dateutil.parser.parse(time_)
    except ValueError as e:
        logging.debug("Unable to parse %s as a date time" % time_)
        logging.debug(str(e))
        return (None, None)
    timezone = dateTime.tzinfo
    if timezone is None: #got a naive time, so assume is local time
        local_dateTime = dateTime.replace(tzinfo=tzlocal())
    elif timezone == tzutc(): #got a zulu time
        local_dateTime = dateTime.astimezone(tzlocal()) #datetime with localtime offset (from OS)
    else:
        local_dateTime = dateTime #use datetime as supplied
    utc_dateTime = local_dateTime.astimezone(tzutc()) #datetime with 00:00 offset
    return (utc_dateTime,local_dateTime)

class Date:
    def __init__(self, calendar=None):
        self.calendar = calendar

    def getDate(self):
        #hack for the gtk calendar widget
        if self.calendar is not None:
            year,month,day = self.calendar.get_date()
            # Selected day might be larger than current month's number of days.
            # Iterate backwards until we find valid date.
            while day >= 1:
                try:
                    # months from gtk calendar are numbered from 0 to 11
                    return datetime.date(year, month + 1, day)
                except ValueError :
                    day -= 1
            raise ValueError("Invalid date supplied: "
                             "day is before 1st of month.")
        else:
            return datetime.date.today()

    def setDate(self,newdate):
        year,month,day = newdate.split("-")
        self.calendar.select_month(int(month)-1,int(year))
        self.calendar.select_day(int(day))

class DateRange(object):

    """A date range consisting of a start date and an end date."""

    def __init__(self, start_date, end_date):
        if not isinstance(start_date, datetime.date):
            raise TypeError("Start date must be datetime.date, not {0}.".format(type(start_date).__name__))
        if not isinstance(end_date, datetime.date):
            raise TypeError("End date must be datetime.date, not {0}.".format(type(start_date).__name__))
        if start_date > end_date:
            raise ValueError("End date cannot be before start date.")
        self._start_date = start_date
        self._end_date = end_date

    @property
    def start_date(self):
        return self._start_date

    @property
    def end_date(self):
        return self._end_date

    def __str__(self):
        fmt = "%Y%m%d"
        return self._start_date.strftime(fmt) + "-" + self._end_date.strftime(fmt)

    @classmethod
    def for_week_containing(clazz, date):
        """Get the date range for the week containing the given date.

        The date range will start on the first day of the week and end on the
        last day of the week. The week start and end days are locale dependent.

        Args:
            date (datetime.date): a date within the week to get the date range for.
        Returns:
            (DateRange): the date range for a week.
        """
        day_of_week = (int(date.strftime("%w")) - get_platform().get_first_day_of_week()) % 7
        date_start = date + datetime.timedelta(days = 0 - day_of_week)
        date_end = date + datetime.timedelta(days = 6 - day_of_week)
        return DateRange(date_start, date_end)

    @classmethod
    def for_month_containing(clazz, date):
        """Get the date range for the month containing the given date.

        The date range will start on the first day of the month and end on the
        last day of the month.

        Args:
            date (datetime.date): a date within the month to get the date range for.
        Returns:
            (DateRange): the date range for a month.
        """
        date_start = datetime.date(date.year, date.month, 1)
        next_month = date.month + 1
        next_month_year = date.year
        if (next_month == 13):
            next_month = 1
            next_month_year += 1
        date_end = datetime.date(next_month_year, next_month, 1) - datetime.timedelta(days=1)
        return DateRange(date_start, date_end)

    @classmethod
    def for_year_containing(clazz, date):
        """Get the date range for the year containing the given date.

        The date range will start on the first day of the year and end on the
        last day of the year.

        Args:
            date (datetime.date): a date within the year to get the date range for.
        Returns:
            (DateRange): the date range for a year.
        """
        date_start = datetime.date(date.year, 1, 1)
        date_end = datetime.date(date.year, 12, 31)
        return DateRange(date_start, date_end)



#!/usr/bin/env python

import os, sys
from optparse import OptionParser
from lxml import etree

def main():
    #Setup usage and permitted options
    usage = "usage: %prog [options] filename"
    parser = OptionParser(usage=usage)
    parser.add_option("-s", "--schema",
    dest="schema",
    help="specify schema to validate FILE against")
    parser.add_option("-v", "--verbose",
            action="store_true", dest="verbose", default=False,
            help="show extra information (including why validation fails)")
    (options, args) = parser.parse_args()
    xmlfiles = []
    for arg in args:
        if not os.path.isfile(arg):
            print("File %s does not exist. Removing from list" % arg)
        else:
            xmlfiles.append(arg)
    #Ensure that at least one file has been supplied to validate
    if len(xmlfiles) < 1:
        parser.error("must supply at least one file to validate")
    #If a specific schema has been passed, ensure the file exists
    if options.schema:
        if os.path.isfile(options.schema):
            schemas = [options.schema]
        else:
            parser.error("Schema specified must exist")
    else:
        schemas = ["Topografix_gpx11.xsd", "Cluetrust_gpxdata10.xsd", "GarminTrainingCenterDatabase_v1.xsd", "GarminTrainingCenterDatabase_v2.xsd"]

    for schema in schemas:
        print("Validating against schema: %s" % schema)
        #Parse XML schema
        xmlschema_doc = etree.parse(schema)
        xmlschema = etree.XMLSchema(xmlschema_doc)
        #Load an XML file and try to validate it
        for xmlfile in xmlfiles:
            xmldoc = etree.parse(xmlfile)
            if (xmlschema.validate(xmldoc)):
                print("\033[0;32mFile %s validates against %s\033[m" % (xmlfile, schema))
            else:
                print("\033[0;31mFile %s does not validate against %s\033[m" % (xmlfile, schema))
                if options.verbose:
                    log = xmlschema.error_log
                    error = log.last_error
                    print(error)

if __name__ == "__main__":
    main()
